/*-
 * #%L
 * Proof Utility Library
 * $Id:$
 * $HeadURL:$
 * %%
 * Copyright (C) 2014 - 2017 Live Ontologies Project
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */
package org.liveontologies.puli;

import java.util.Collection;
import java.util.Set;

import com.google.common.base.Preconditions;

class AddAssertedProofNode<C> extends ConvertedProofNode<C> {

	private final Set<? extends C> assertedConclusions_;

	AddAssertedProofNode(ProofNode<C> delegate,
			Set<? extends C> assertedConclusions) {
		super(delegate);
		Preconditions.checkNotNull(assertedConclusions);
		this.assertedConclusions_ = assertedConclusions;
	}

	@Override
	public Collection<ProofStep<C>> getInferences() {
		Collection<ProofStep<C>> result = super.getInferences();
		if (assertedConclusions_.contains(getMember())) {
			result.add(new AssertedProofStep<C>(this));
		}
		return result;
	}

	@Override
	protected final void convert(ConvertedProofStep<C> step) {
		convert(new AddAssertedProofStep<C>(step.getDelegate(),
				assertedConclusions_));
	}

	void convert(AddAssertedProofStep<C> step) {
		super.convert(step);
	}

	/** implementation of {@link ProofStep} */

}
